package edu.lawrence.canvasdemo

import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.ui.geometry.Offset
import androidx.lifecycle.ViewModel

class GraphModel : ViewModel() {
    var vertices = mutableStateListOf<Vertex>()
    var edges = mutableStateListOf<Edge>()
    var dragging = mutableStateOf<Vertex?>(null)
    var start = mutableStateOf<Vertex?>(null)

    init {
        val x1 = 200.0f
        val x2 = 400.0f
        val v1 = Vertex(centerX = x1,centerY = x1)
        val v2 = Vertex(centerX = x2,centerY = x2)
        vertices.add(v1)
        vertices.add(v2)
        edges.add(Edge(origin = v1, dest = v2))
    }

    fun makeVertex(center : Offset) {
        vertices.add(Vertex(centerX = center.x,centerY = center.y))
    }

    fun startDrag(pt:Offset) {
        for (v in vertices) {
            if(v.containsPoint(pt)) {
                dragging.value = v
            }
        }
    }

    fun continueDrag(pt:Offset) {
        if(dragging.value != null) {
            val v = dragging.value!!
            v.centerX = pt.x
            v.centerY = pt.y
        }
    }

    fun endDrag(pt:Offset) {
        dragging.value = null
    }

    fun startDraw(pt:Offset) {
        for (v in vertices) {
            if(v.containsPoint(pt)) {
                start.value = v
            }
        }
    }

    fun endDraw(pt:Offset) {
        if(start.value != null) {
            val s = start.value!!
            for (v in vertices) {
                if(v.containsPoint(pt) && v != s) {
                    val newEdge = Edge(origin = s,dest = v)
                    edges.add(newEdge)
                    break
                }
            }
        }
        start.value = null
    }
}